/*****************************************************************************
 *  File: reconfig_utils.h
 *
 *  This file contains the constants, structure definitions, and funtion
 *  prototypes for the application selector.
 *
 ****************************************************************************/

#ifndef APP_SELECTOR_H_
#define APP_SELECTOR_H_

#include "sys/alt_flash_dev.h"

//#define DEBUG_MODE 1 // debugging help
//#define MAN_DEBUG 1 // debugging help
//#define UPDATE_PFL 1 // option to keep or update PFL OPTION BITS
//#define WEB_PAGE_ONLY_MODE 1

#define NUM_VIDEO_FRAMES 2

#define SREC_BUFFER_SIZE             0x00020000
#define FLASH_SECTOR_SIZE            0x40000  // 256KB
#define FLASH_BOOT_BLOCK_SECTOR_SIZE 0x40000  // 256KB
#define HW_IMAGE_SIZE                43*1024*1024
#define SECTORS_FOR_HW_IMAGE         (HW_IMAGE_SIZE / FLASH_SECTOR_SIZE) // 43 * 4 = 172 SEC
#define BYTES_FOR_HW_IMAGE           (SECTORS_FOR_HW_IMAGE * FLASH_SECTOR_SIZE) // 44032 KB
#define DEVICE_HTILE
#ifdef DEVICE_HTILE
#define BOARD_ID 					 0x32     // 0x32 used for board ID in flash
#else
#define BOARD_ID 					 0x2E     // 0x2E used for board ID in flash
#endif
/****************************************************************
// Stratix10 PCIe DevKits
 Section                    Size(KB)    Address
 ---------------------------------------------------------------
 Board test system scratch	512			0x07500000 - 0x0757FFFF
 User software		        14,336		0x06700000 - 0x074FFFFF
 Factory software		    8,192		0x05F00000 - 0x066FFFFF
 Zips(html, web content)    8,192		0x05700000 - 0x05EFFFFF
 User hardware1			    44,032		0x02C00000 - 0x056FFFFF
 Factory hardware		    44,032		0x00100000 - 0x02BFFFFF
 --------------------------------------------------------------
 PFL option bits		    256			0x000C0000 - 0x000FFFFF
 Board information		    256			0x00080000 - 0x000BFFFF
 Ethernet option bits	    256			0x00040000 - 0x0007FFFF
 User design reset vector   256			0x00000000 - 0x0003FFFF
 ---------------------------------------------------------------
 Section                    Size(KB)    Address
 ---------------------------------------------------------------
 User hardware2			    44,032		0x00100000 - 0x02BFFFFF
 --------------------------------------------------------------
 PFL option bits		    256			0x000C0000 - 0x000FFFFF
 Reserved		            256			0x00080000 - 0x000BFFFF
 Reserved	                256			0x00040000 - 0x0007FFFF
 Reserved                   256			0x00000000 - 0x0003FFFF
****************************************************************/

#define FACTORY_HW_IMAGE_PAGE        0
#define FACTORY_HW_IMAGE_OFFSET      0x00100000
#define FACTORY_SW_IMAGE_OFFSET      0x05F00000
#define FACTORY_WP_IMAGE_OFFSET      0x05700000

// correct user #defines
#define USER_HW1_IMAGE_PAGE          1
#define USER_HW2_IMAGE_PAGE          2
#define USER_HW1_IMAGE_OFFSET        0x02C00000
#define USER_HW2_IMAGE_OFFSET        0x00100000
#define USER_SW_IMAGE_OFFSET         0x06700000


#define RO_ZIPFS_OFFSET              0x05700000
#define ETHERNET_OPTION_BITS         0x00040000
#define BOARD_INFO_OFFSET            0x00080000
#define PFL_OPTION_BIT_OFFSET        0x000C0000

// These are the SIVGX SI 16bit masks for the MAXII reconfiguation 16bit registers:
#define MAXII_CLEAR_PSR_MASK         0xFFFE // 1111 1111 1111 1110 | 0
#define MAXII_CLEAR_PSO_MASK         0xFFFD // 1111 1111 1111 1101 | 1
#define MAXII_FORCE_RECONFIG_MASK    0x000B // 0000 0000 0000 1011 | B
#define MAXII_CLEAR_PSS_MASK         0xFFF7 // 1111 1111 1111 0111 | 3

//#define MAXII_RSU_REG				0x6 // was 0x8
#define MAXII_RSU_REG  0x8 			// MAXII_REG2 is 0x8

#define PAGE_USER				1	// NIOS_PIO_PAGE_SELECT_BASE
#define SRST					0	// NIOS_PIO_RECONFIGURE_N_BASE

#define USE_HAL_FLASH_DRIVER            0
#define USE_INTEL_P30_FLASH_DRIVER      0
#define USE_INTEL_G18_FLASH_DRIVER      1

// Error Codes
#define SUCCESS                      0
#define FAILURE                     -1
#define ERROR_FAT_FAILED_TO_MOUNT   -2
#define ERROR_FAT_COULD_NOT_FIND    -3
#define ERROR_FAT_READ_ZERO_BYTES   -4
#define ERROR_SREC_HAS_NO_TIMESTAMP -5
#define ERROR_COULD_NOT_ALLOCATE    -6

// Filetypes
#define FLASH                        1
#define RBF                          2

#define FILE_BUFFER_SIZE FLASH_SECTOR_SIZE

int ReconfigFPGA(unsigned int hw_flash_page);
void InitializePFLOptionBits(unsigned int option_bit_offset);

#endif /*APP_SELECTOR_H_*/
