module  bts_config(
        input            clk_50m_s10        , // System clock, 50MHz
        input            cpu_resetn         , // FPGA reset, low active
		
        output  [3:0]    user_led_g         , // User LEDs
        output  [3:0]    user_led_r         , // User LEDs
        input   [3:0]    user_dip           , // User DIP switches, user_dip[7]: SW4.1 -> s10_unlock
        input   [2:0]    user_pb            , // User push buttons

        inout            i2c_1v8_scl        , // SCL
        inout            i2c_1v8_sda        , // SDA

        output  [26:0]   fm_a               , // CFI Flash interface
        inout   [15:0]   fm_d               , // CFI Flash interface
        output           flash_wen          , // CFI Flash interface
        output           flash_oen          , // CFI Flash interface
        output  [1:0]    flash_cen          , // CFI Flash interface
        output           flash_advn         , // CFI Flash interface
        output           flash_clk          , // CFI Flash interface
        output           flash_resetn       , // CFI Flash interface
        input   [1:0]    flash_rdybsyn        // CFI Flash interface
        );

wire    flash_cen_temp;
wire    flash_sel;

assign  flash_cen[0]    = flash_cen_temp | flash_sel;
assign  flash_cen[1]    = flash_cen_temp | (!flash_sel);
assign  flash_advn      = 1'b0;
assign  flash_clk       = 1'b0;

q_sys   u0_q_sys (
        .clk_clk                                     (clk_50m_s10   ), //                                  clk.clk
        .reset_reset_n                               (cpu_resetn    ), //                                reset.reset_n
        .opencores_i2c_0_export_0_scl_pad_io         (i2c_1v8_scl   ), //             opencores_i2c_0_export_0.scl_pad_io
        .opencores_i2c_0_export_0_sda_pad_io         (i2c_1v8_sda   ), //                                     .sda_pad_io
        .tb_fm_out_tb_fm_address                     (fm_a          ), //                            tb_fm_out.tb_fm_address
        .tb_fm_out_tb_fm_data                        (fm_d          ), //                                     .tb_fm_data
        .tb_fm_out_flash_csn                         (flash_cen_temp), //                                     .flash_csn
        .tb_fm_out_flash_ren                         (flash_oen     ), //                                     .flash_ren
        .tb_fm_out_flash_wen                         (flash_wen     ), //                                     .flash_wen
        .tb_fm_out_flash_resetn                      (flash_resetn  ), //                                     .flash_resetn
        .pio_0_user_led_external_connection_export   ({user_led_g,user_led_r}      ), //   pio_0_user_led_external_connection.export
        .pio_2_user_pb_external_connection_export    (user_pb       ), //    pio_2_user_pb_external_connection.export
        .pio_3_flash_sel_external_connection_export  (flash_sel     ),
		.pio_1_user_dipsw_external_connection_export (user_dip      )  // pio_1_user_dipsw_external_connection.export
        );

endmodule
