
`timescale 1 ps / 1 ps
module  ddr3 (
        //Reset&Clock
        input           cpu_resetn          , // Reset, low active
        input           clk_50m             , // System clock, 50MHz

        //--------user_led_dip_pb----------
        output  [3:0]   user_led_g          , // User LEDs
        output  [3:0]   user_led_r          , // User LEDs
        input   [3:0]   user_dipsw          , // User DIP Switches (TR=0)
        input   [2:0]   user_pb             , // User Push Buttons (TR=0)

        //EMIF interface
        input           clk_hilo_p          , // EMIF reference clock
        //input         clk_hilo_n          , // EMIF reference clock

        output          mem_clk_p           , // 
        output          mem_clk_n           , // 
        //output[31:0]  mem_addr_cmd        , // MEM_ADDR_CMD: Address/ODT/CSn/CASn/CKE/BA/WEn/RESETn
        output  [28:0]  mem_addr_cmd        , // MEM_ADDR_CMD: Address/ODT/CSn/CASn/CKE/BA/WEn/RESETn

        //inout [33:0]  mem_dqa             , // MEM_DQA
        inout	[31:0]	mem_dqa             , // MEM_DQA
        inout   [3:0]   mem_dqsa_p          , // MEM_DQSA_P
        inout   [3:0]   mem_dqsa_n          , // MEM_DQSA_N
        //inout [1:0]   mem_qka_p           , // MEM_QKA_P, NOT used for DDR3/DDR4
        output  [3:0]   mem_dma             , // MEM_DMA

        //inout [33:0]  mem_dqb             , // MEM_DQB
        inout   [31:0]  mem_dqb             , // MEM_DQB
        inout   [3:0]   mem_dqsb_p          , // MEM_DQSB_P
        inout   [3:0]   mem_dqsb_n          , // MEM_DQSB_N
        //inout [1:0]   mem_qkb_p           , // MEM_QKB_P, NOT used for DDR3/DDR4
        output  [3:0]   mem_dmb             , // MEM_DMB

        inout           mem_dqs_addr_cmd_p  , // MEM_DQS_ADDR_CMD_P
        inout           mem_dqs_addr_cmd_n  , // MEM_DQS_ADDR_CMD_N

        //inout	[8:0]	mem_dq_addr_cmd     , // MEM_DQ_ADDR_CMD
        inout   [8:1]   mem_dq_addr_cmd     , // MEM_DQ_ADDR_CMD
        output          mem_dq_addr_cmd0    , // MEM_DQ_ADDR_CMD0

        input           rzq_b2m               // RZQ
        );

reg     [26:0] heart_beat_cnt;
wire    [2:0]  test_status;

//glue logic for heart beat
always @(posedge clk_50m or negedge cpu_resetn)
        if (!cpu_resetn)
            heart_beat_cnt <= 27'h0; //0x7FFFFFF
        else
            heart_beat_cnt <= heart_beat_cnt + 1'b1;

//----------------------------------------------------------------------------------------------------------
// Note: Make sure assign '0' to the physical Data Mask pin if DM is not enabled in ddr3 IP core
//----------------------------------------------------------------------------------------------------------
ed_synth    u0_ed_synth (
            .clk_50m_clk                              (clk_50m                                               ), //                            clk_50.clk
            .global_reset_reset_n                     (cpu_resetn                                            ), //                      global_reset.reset_n
            .emif_s10_0_mem_mem_ck                    (mem_clk_p                                             ), //                    emif_s10_0_mem.mem_ck
            .emif_s10_0_mem_mem_ck_n                  (mem_clk_n                                             ), //                                  .mem_ck_n
            .emif_s10_0_mem_mem_a                     (mem_addr_cmd[14:0]                                    ), //                                  .mem_a
            .emif_s10_0_mem_mem_ba                    (mem_addr_cmd[18:16]                                   ), //                                  .mem_ba
            .emif_s10_0_mem_mem_cke                   (mem_addr_cmd[20]                                      ), //                                  .mem_cke
            .emif_s10_0_mem_mem_cs_n                  (mem_addr_cmd[22]                                      ), //                                  .mem_cs_n
            .emif_s10_0_mem_mem_odt                   (mem_addr_cmd[24]                                      ), //                                  .mem_odt
            .emif_s10_0_mem_mem_reset_n               (mem_addr_cmd[27]                                      ), //                                  .mem_reset_n
            .emif_s10_0_mem_mem_we_n                  (mem_addr_cmd[28]                                      ), //                                  .mem_we_n
            .emif_s10_0_mem_mem_ras_n                 (mem_addr_cmd[26]                                      ), //                                  .mem_ras_n
            .emif_s10_0_mem_mem_cas_n                 (mem_addr_cmd[19]                                      ), //                                  .mem_cas_n
            .emif_s10_0_mem_mem_dqs                   ({mem_dqs_addr_cmd_p, mem_dqsb_p[3:0], mem_dqsa_p[3:0]}), //                                  .mem_dqs
            .emif_s10_0_mem_mem_dqs_n                 ({mem_dqs_addr_cmd_n, mem_dqsb_n[3:0], mem_dqsa_n[3:0]}), //                                  .mem_dqs_n
            .emif_s10_0_mem_mem_dq                    ({mem_dq_addr_cmd[8:1], mem_dqb[31:0], mem_dqa[31:0]}  ), //                                  .mem_dq
            .emif_s10_0_mem_mem_dm                    ({mem_dq_addr_cmd0, mem_dmb[3:0], mem_dma[3:0]}        ), //                                  .mem_dm
            .emif_s10_0_oct_oct_rzqin                 (rzq_b2m                                               ), //                    emif_s10_0_oct.oct_rzqin
            .ddr_ref_clk_clk                          (clk_hilo_p                                            ), //                       ddr_ref_clk.clk
            .local_reset_req                          (cpu_resetn                                            ), //                   local_reset_req.local_reset_req
            .local_reset_done                         (test_status[2]                                        ), //                local_reset_status.local_reset_done
            .master_driver_msgdma_0_error_mon_export  (test_status[1]                                        ), //  master_driver_msgdma_0_error_mon.export
            .master_driver_msgdma_0_status_mon_export (test_status[0]                                        )  // master_driver_msgdma_0_status_mon.export
			);

assign mem_addr_cmd[15] = 1'b0;
assign mem_addr_cmd[21] = 1'b0;
assign mem_addr_cmd[23] = 1'b0;
assign mem_addr_cmd[25] = 1'b0;

assign user_led_g = {heart_beat_cnt[26], heart_beat_cnt[26], heart_beat_cnt[26], test_status[1]};
assign user_led_r = {test_status[2], 2'b11, test_status[0]};

endmodule
