
module qts_fmca (

//---------------clk---------------
    input           clk_fpga_100m,
    input           clk_50,

//---------------i2c---------------  
    output          lt_io_scl,
    inout           lt_io_sda,

//------------cpu_reset------------
    input           cpu_resetn,
 
//--------user_led_dip_pb----------
    output [ 3:0]   user_led_g,
    output [ 3:0]   user_led_r,
    input  [ 3:0]   user_dipsw,
    input  [ 2:0]   user_pb, 
  
//---------------ref---------------  
    input           refclk_fmca_p,
 
//---------------fmc---------------               
    output [15:0]   fmca_dp_c2m_p, 
    input  [15:0]   fmca_dp_m2c_p, 
 
    input  [14:0]   fmca_la_rx_n, 
    input  [14:0]   fmca_la_rx_p, 
    output [14:0]   fmca_la_tx_n, 
    output [14:0]   fmca_la_tx_p, 
 
    input           fmca_la_tx_n_15, 
    input           fmca_la_tx_p_15, 
    output          fmca_la_tx_n_16, 
    output          fmca_la_tx_p_16, 

    input           fmca_prsntn,
    inout           fmca_scl,
    inout           fmca_sda
);

    wire  [31:0]    fmca_cmos_tx_parallel_data;
    wire  [31:0]    fmca_cmos_rx_parallel_data;
    wire  [31:0]    fmca_cmos_parallel_debug;
    wire  [ 1:0]    fmca_cmos_loopback_en;
    
    wire            fmca_xcvr_atx_pll_a10_pll_locked;
    wire  [ 5:0]    fmca_xcvr_bank_4c_rx_is_lockedtoref;
    wire  [ 5:0]    fmca_xcvr_bank_4d_rx_is_lockedtoref;
    wire  [ 3:0]    fmca_xcvr_bank_4e_rx_is_lockedtoref;
    
    reg   [26:0]    heart_beat_cnt;

//glue logic for heart beat
    always @(posedge clk_50 or negedge cpu_resetn)
        if (!cpu_resetn)
            heart_beat_cnt <= 27'h0; //0x7FFFFFF
        else
            heart_beat_cnt <= heart_beat_cnt + 1'b1;
    
    assign  {
                fmca_la_tx_n_16, fmca_la_tx_n[14:0],
                fmca_la_tx_p_16, fmca_la_tx_p[14:0]
            } = fmca_cmos_loopback_en[1] ? fmca_cmos_parallel_debug : fmca_cmos_tx_parallel_data;   
      
    assign  fmca_cmos_rx_parallel_data = fmca_cmos_loopback_en[0] ? fmca_cmos_tx_parallel_data :  {
                                                                                                      fmca_la_tx_n_15, fmca_la_rx_n[14:0],
                                                                                                      fmca_la_tx_p_15, fmca_la_rx_p[14:0]
                                                                                                  };
    
    fmc_xcvr_system fmc_xcvr_system_i 
    (
        .clk_100_clk                                                                 (clk_fpga_100m                      ), //                                                     clk_100.clk
        .clk_100_reset_reset_n                                                       (cpu_resetn                         ), //                                               clk_100_reset.reset_n
        .clk_50_clk                                                                  (clk_50                             ), //                                                      clk_50.clk
        .clk_50_reset_reset_n                                                        (cpu_resetn                         ), //                                                clk_50_reset.reset_n
        .atx_pll_4c_refclk_in_clk_clk                                                (refclk_fmca_p                      ), //                                    atx_pll_4c_refclk_in_clk.clk                                   atx_pll_4n_refclk_in_clk.clk
        .fmc_xcvr_system_pll_status_pll_locked_output_pll_locked                     (fmca_xcvr_atx_pll_a10_pll_locked   ), //                    xcvr_atx_pll_4c_locked_pll_locked_output.pll_locked                   xcvr_atx_pll_4n_locked_pll_locked_output.pll_locked
        .fmc_cmos_system_0_nativephy_loopback_cont_0_rx_seriallpbken_rx_seriallpbken (fmca_cmos_loopback_en              ), // fmc_cmos_system_0_nativephy_loopback_cont_0_rx_seriallpbken.rx_seriallpbken
        .fmc_cmos_system_0_pio_0_external_connection_export                          (fmca_cmos_parallel_debug           ), //                 fmc_cmos_system_0_pio_0_external_connection.export
        .fmc_cmos_system_0_xcvr_st_converter_0_rx_parallel_data_rx_parallel_data     (fmca_cmos_rx_parallel_data         ), //      fmc_cmos_system_0_xcvr_st_converter_0_rx_parallel_data.rx_parallel_data
        .fmc_cmos_system_0_xcvr_st_converter_0_tx_parallel_data_tx_parallel_data     (fmca_cmos_tx_parallel_data         ), //      fmc_cmos_system_0_xcvr_st_converter_0_tx_parallel_data.tx_parallel_data
        .fmc_xcvr_system_bank_4c_0_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[0]                   ), //  fmc_xcvr_system_bank_4c_0_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4c_0_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[0]                   ), //  fmc_xcvr_system_bank_4c_0_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4c_1_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[1]                   ), //  fmc_xcvr_system_bank_4c_1_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4c_1_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[1]                   ), //  fmc_xcvr_system_bank_4c_1_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4c_2_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[2]                   ), //  fmc_xcvr_system_bank_4c_2_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4c_2_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[2]                   ), //  fmc_xcvr_system_bank_4c_2_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4c_3_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[3]                   ), //  fmc_xcvr_system_bank_4c_3_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4c_3_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[3]                   ), //  fmc_xcvr_system_bank_4c_3_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4c_4_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[4]                   ), //  fmc_xcvr_system_bank_4c_4_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4c_4_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[4]                   ), //  fmc_xcvr_system_bank_4c_4_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4c_5_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[5]                   ), //  fmc_xcvr_system_bank_4c_5_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4c_5_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[5]                   ), //  fmc_xcvr_system_bank_4c_5_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4d_0_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[6]                   ), //  fmc_xcvr_system_bank_4d_0_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4d_0_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[6]                   ), //  fmc_xcvr_system_bank_4d_0_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4d_1_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[7]                   ), //  fmc_xcvr_system_bank_4d_1_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4d_1_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[7]                   ), //  fmc_xcvr_system_bank_4d_1_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4d_2_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[8]                   ), //  fmc_xcvr_system_bank_4d_2_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4d_2_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[8]                   ), //  fmc_xcvr_system_bank_4d_2_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4d_3_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[9]                   ), //  fmc_xcvr_system_bank_4d_3_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4d_3_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[9]                   ), //  fmc_xcvr_system_bank_4d_3_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4d_4_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[10]                  ), //  fmc_xcvr_system_bank_4d_4_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4d_4_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[10]                  ), //  fmc_xcvr_system_bank_4d_4_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4d_5_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[11]                  ), //  fmc_xcvr_system_bank_4d_5_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4d_5_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[11]                  ), //  fmc_xcvr_system_bank_4d_5_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4e_0_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[12]                  ), //  fmc_xcvr_system_bank_4e_0_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4e_0_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[12]                  ), //  fmc_xcvr_system_bank_4e_0_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4e_1_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[13]                  ), //  fmc_xcvr_system_bank_4e_1_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4e_1_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[13]                  ), //  fmc_xcvr_system_bank_4e_1_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4e_2_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[14]                  ), //  fmc_xcvr_system_bank_4e_2_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4e_2_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[14]                  ), //  fmc_xcvr_system_bank_4e_2_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .fmc_xcvr_system_bank_4e_3_xcvr_native_s10_0_rx_serial_data_rx_serial_data   (fmca_dp_m2c_p[15]                  ), //  fmc_xcvr_system_bank_4e_3_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .fmc_xcvr_system_bank_4e_3_xcvr_native_s10_0_tx_serial_data_tx_serial_data   (fmca_dp_c2m_p[15]                  )  //  fmc_xcvr_system_bank_4e_3_xcvr_native_s10_0_tx_serial_data.tx_serial_data
    ); 

    assign user_led_g[0]   =  !fmca_xcvr_atx_pll_a10_pll_locked;
    assign user_led_g[1]   =  1'b1;
    assign user_led_g[2]   =  1'b1;
    assign user_led_g[3]   =  heart_beat_cnt[26];
    
    assign user_led_r      =  4'hf;
    
    assign lt_io_scl       =  1'bz;
    assign lt_io_sda       =  1'bz;
    
    assign fmca_scl        =  1'bz;
    assign fmca_sda        =  1'bz;

endmodule
