module qts_pcie (

//---------------clk---------------
    input           clk_fpga_100m,
    input           clk_50,

//---------------i2c---------------  
    output          lt_io_scl,
    inout           lt_io_sda,

//------------cpu_reset------------
    input           cpu_resetn,
 
//--------user_led_dip_pb----------
    output [ 3:0]   user_led_g,
    output [ 3:0]   user_led_r,
    input  [ 3:0]   user_dipsw,
    input  [ 2:0]   user_pb,

//---------------ref---------------
    input           refclk_pcie_p,
  
//---------------pcie--------------      
    input  [15:0]   pcie_rx_p,
    output [15:0]   pcie_tx_p, 
    input           pcie_perstn,
    input           pcie_smbclk,
    inout           pcie_smbdat,
    input           pcie_waken,  
    output          pcie_led_x1,
    output          pcie_led_x4,
    output          pcie_led_x8,
    output          pcie_led_x16,  
    output          pcie_led_g2,
    output          pcie_led_g3
);
 
    wire            pcie_xcvr_atx_pll_locked;
 
    reg   [26:0]    heart_beat_cnt;
 
//glue logic for heart beat
    always @(posedge clk_50 or negedge cpu_resetn)
        if (!cpu_resetn)
            heart_beat_cnt <= 27'h0;
        else
            heart_beat_cnt <= heart_beat_cnt + 1'b1;

//qsys project instance 
    pcie_xcvr_system pcie_xcvr_system_i (
        .atx_pll_1c_refclk_in_clk_clk                                               (refclk_pcie_p           ), //                                    atx_pll_1c_refclk_in_clk.clk
        .clk_100_clk                                                                (clk_fpga_100m           ), //                                                     clk_100.clk
        .clk_100_reset_reset_n                                                      (cpu_resetn              ), //                                               clk_100_reset.reset_n
        .clk_50_clk                                                                 (clk_50                  ), //                                                      clk_50.clk
        .clk_50_reset_reset_n                                                       (cpu_resetn              ), //                                                clk_50_reset.reset_n
        .pcie_xcvr_system_bank_1c_0_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[0]            ), // pcie_xcvr_system_bank_1c_0_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1c_0_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[0]            ), // pcie_xcvr_system_bank_1c_0_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1c_1_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[1]            ), // pcie_xcvr_system_bank_1c_1_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1c_1_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[1]            ), // pcie_xcvr_system_bank_1c_1_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1c_2_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[2]            ), // pcie_xcvr_system_bank_1c_2_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1c_2_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[2]            ), // pcie_xcvr_system_bank_1c_2_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1c_3_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[3]            ), // pcie_xcvr_system_bank_1c_3_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1c_3_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[3]            ), // pcie_xcvr_system_bank_1c_3_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1c_4_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[4]            ), // pcie_xcvr_system_bank_1c_4_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1c_4_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[4]            ), // pcie_xcvr_system_bank_1c_4_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1c_5_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[5]            ), // pcie_xcvr_system_bank_1c_5_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1c_5_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[5]            ), // pcie_xcvr_system_bank_1c_5_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1d_0_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[6]            ), // pcie_xcvr_system_bank_1d_0_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1d_0_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[6]            ), // pcie_xcvr_system_bank_1d_0_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1d_1_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[7]            ), // pcie_xcvr_system_bank_1d_1_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1d_1_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[7]            ), // pcie_xcvr_system_bank_1d_1_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1d_2_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[8]            ), // pcie_xcvr_system_bank_1d_2_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1d_2_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[8]            ), // pcie_xcvr_system_bank_1d_2_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1d_3_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[9]            ), // pcie_xcvr_system_bank_1d_3_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1d_3_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[9]            ), // pcie_xcvr_system_bank_1d_3_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1d_4_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[10]           ), // pcie_xcvr_system_bank_1d_4_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1d_4_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[10]           ), // pcie_xcvr_system_bank_1d_4_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1d_5_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[11]           ), // pcie_xcvr_system_bank_1d_5_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1d_5_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[11]           ), // pcie_xcvr_system_bank_1d_5_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1e_0_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[12]           ), // pcie_xcvr_system_bank_1e_0_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1e_0_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[12]           ), // pcie_xcvr_system_bank_1e_0_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1e_1_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[13]           ), // pcie_xcvr_system_bank_1e_1_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1e_1_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[13]           ), // pcie_xcvr_system_bank_1e_1_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1e_2_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[14]           ), // pcie_xcvr_system_bank_1e_2_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1e_2_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[14]           ), // pcie_xcvr_system_bank_1e_2_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_bank_1e_3_xcvr_native_s10_0_rx_serial_data_rx_serial_data (pcie_rx_p[15]           ), // pcie_xcvr_system_bank_1e_3_xcvr_native_s10_0_rx_serial_data.rx_serial_data
        .pcie_xcvr_system_bank_1e_3_xcvr_native_s10_0_tx_serial_data_tx_serial_data (pcie_tx_p[15]           ), // pcie_xcvr_system_bank_1e_3_xcvr_native_s10_0_tx_serial_data.tx_serial_data
        .pcie_xcvr_system_pll_status_pll_locked_output_pll_locked                   (pcie_xcvr_atx_pll_locked)  //               pcie_xcvr_system_pll_status_pll_locked_output.pll_locked
    );  

  assign pcie_led_x1     =  1'b0;
  assign pcie_led_x4     =  1'b0;
  assign pcie_led_x8     =  1'b0;
  assign pcie_led_x16    =  1'b0;
  assign pcie_led_g2     =  1'b0;
  assign pcie_led_g3     =  1'b0;    
    
  assign lt_io_scl       =  1'bz;
  assign lt_io_sda       =  1'bz;

  assign user_led_g[0]   =  !pcie_xcvr_atx_pll_locked;
  assign user_led_g[1]   =  1'b1;
  assign user_led_g[2]   =  1'b1;
  assign user_led_g[3]   =  heart_beat_cnt[26];
 
  assign user_led_r[3:0] =  4'hFF;
 
endmodule
