#!/bin/bash

if [ $# != 2 ] ; then
	echo "Usage:"
	echo "------"
	echo "    sh build_hw.sh <input sof/pof file> <page number>"
	echo " "
	echo "Example:"
	echo "    sh build_hw.sh web_server.sof 0"
	exit
fi

file=$1
filename=${file%.*}
extension=${file##*.}

if [ $extension != "sof" ] && [ $extension != "pof" ] ; then
	echo "Usage:"
	echo "------"
	echo "    sh build_hw.sh <input sof/pof file> <page number>"
	exit
fi

if [ $2 == "0" ] ; then
	startaddr="00100000"
	username="Page_0"
	pageflags="1"
	offset=1048576
elif [ $2 == "1" ]; then
	startaddr="02C00000"
	username="Page_1"
	pageflags="2"
	offset=46137344
else
	startaddr="00100000"
	username="Page_0"
	pageflags="1"
	offset=1048576
fi

touch ${filename}".cof"

if [ $extension == "sof" ] ; then
	echo "<?xml version=\"1.0\" encoding=\"US-ASCII\" standalone=\"yes\"?>
<cof>
	<eprom_name>CFI_1GB</eprom_name>
	<output_filename>"${filename}".pof</output_filename>
	<n_pages>1</n_pages>
	<width>1</width>
	<mode>21</mode>
	<sof_data>
		<start_address>"${startaddr}"</start_address>
		<user_name>"${username}"</user_name>
		<page_flags>"${pageflags}"</page_flags>
		<bit0>
			<sof_filename>"${file}"</sof_filename> 
		</bit0>
	</sof_data>
	<version>10</version>
	<create_cvp_file>0</create_cvp_file>
	<create_hps_iocsr>0</create_hps_iocsr>
	<auto_create_rpd>0</auto_create_rpd>
	<rpd_little_endian>1</rpd_little_endian>
	<options>
		<map_file>0</map_file>
		<option_start_address>c0000</option_start_address>
		<dynamic_compression>0</dynamic_compression>
	</options>
	<advanced_options>
		<ignore_epcs_id_check>2</ignore_epcs_id_check>
		<ignore_condone_check>2</ignore_condone_check>
		<plc_adjustment>0</plc_adjustment>
		<post_chain_bitstream_pad_bytes>-1</post_chain_bitstream_pad_bytes>
		<post_device_bitstream_pad_bytes>-1</post_device_bitstream_pad_bytes>
		<bitslice_pre_padding>1</bitslice_pre_padding>
	</advanced_options>
</cof>" > ${filename}".cof"

	echo "Converting to Programmer Object File."
	quartus_cpf -c ${filename}.cof
	rm ${filename}.cof
fi

echo "Generating binary file..."
str=`xxd -l 200 -ps -c 200 ${filename}.pof | sed 's/ffffff*//g'` 
head=`expr length $str`
len=`expr $head / 2 + $offset`
dd skip=$len count=45088700 if=${filename}.pof of=${filename}.bin ibs=1 obs=1048576

echo "Object file generated successfully!"
echo "Please choose this file to the Hardware file input box and click upload to update hareware design on CFI flash."