//******************************************************************************
//
//  File Name:  max_pgm_sel.v
//  Version:    1.0
//  Date:       April 28, 2014
//  Model:      image programming selection
//  Function:   Flash Page Selection
//  Author:     mechen@altera.com
//  Company:    Altera
//
//*****************************************************************************/

`timescale 1ns / 100ps

module  max_pgm_sel
        (
        input   wire       i_reset_n         , // reset
        input   wire       i_slow_clk        , // clock
        input   wire       i_pgm_sel_disable , // Page selection disable
        input   wire [3:0] i_page_sel_gui    , // Page selection via GUI
        input   wire       i_factory_load    , // Factory load setting
        input   wire       i_pgm_sel         , // Programming page select
        output  wire [2:0] o_pgm_led         , // LED indication
        output  wire       o_pgm_sel_pb_trig , // Page_sel pushbutton
        output  wire [2:0] o_fpga_pgm          // FPGA programming page selection
        );

//--------------------------------------------------
// Signal defination
//--------------------------------------------------

parameter            FILTER_WIDTH = 14; // even number: 2, 4, 6, 8 ...
parameter            INIT_BIT     = 1 ; // 0: 1'b0; 1: 1'b1;

wire                 pgm_sel          ; // 
reg                  pgm_sel_reg      ; // 
wire                 pgm_sel_neg      ; // 
reg        [1:0]     fpga_pgm_cnt     ; // 
wire       [1:0]     fpga_pgm_cnt_init; // 
reg        [2:0]     pgm_led          ; // 
wire       [2:0]     pgm_led_init     ; // 
reg        [2:0]     fpga_pgm         ; // 
wire       [2:0]     fpga_pgm_init    ; // 

//--------------------------------------------------
// Main function body
//--------------------------------------------------
//Initialization
assign fpga_pgm_cnt_init = i_factory_load ? 2'd1 : 2'd0;
assign pgm_led_init      = i_factory_load ? 3'b101 : 3'b110;
assign fpga_pgm_init     = i_factory_load ? 3'b001 : 3'b000;

// Output assignments
assign o_pgm_led  = pgm_led;
assign o_fpga_pgm = fpga_pgm;
assign o_pgm_sel_pb_trig = pgm_sel_neg;

//---------------------------------------------------------------------------------------
//Flash page selection
//
filter_circuit  #(
                .FILTER_WIDTH (FILTER_WIDTH ), // even number excepts 0, such as 2, 4, 6, 8 ...
                .INIT_BIT     (INIT_BIT     )  // initialization bit, 0 or 1
                )
                filter_circuit_inst
                (
                .i_clk_rst    (!i_reset_n   ), // reset, high valid
                .i_clk        (i_slow_clk   ), // clock
                .i_framein    (i_pgm_sel    ), // input
                .o_frameout   (pgm_sel      )  // output
                );

always @(posedge i_slow_clk)
begin
pgm_sel_reg <= pgm_sel;
end

assign pgm_sel_neg = pgm_sel_reg & (!pgm_sel); // negedge of pgm_sel

always @(posedge i_slow_clk or negedge i_reset_n)
begin
  if(!i_reset_n)
     fpga_pgm_cnt <= fpga_pgm_cnt_init;
  else if(i_pgm_sel_disable)                   // No change during PFL configuration
     fpga_pgm_cnt <= fpga_pgm_cnt;
  else if(pgm_sel_neg)
     begin
	   if(fpga_pgm_cnt == 2'd2)
         fpga_pgm_cnt <= 2'd0;
	   else
         fpga_pgm_cnt <= fpga_pgm_cnt + 1'd1;
     end
  else if(i_page_sel_gui[3])
     fpga_pgm_cnt <= i_page_sel_gui[1:0];      // Valid page selection: 0, 1, 2	 
  else
     fpga_pgm_cnt <= fpga_pgm_cnt;
end

always @(posedge i_slow_clk or negedge i_reset_n)
begin
  if(!i_reset_n)
     begin
     pgm_led  <= pgm_led_init;
     fpga_pgm <= fpga_pgm_init;
     end
  else if(fpga_pgm_cnt == 2'd0)
     begin
     pgm_led  <= 3'b110;
     fpga_pgm <= 3'b000;
     end
  else if(fpga_pgm_cnt == 2'd1)
     begin
     pgm_led  <= 3'b101;
     fpga_pgm <= 3'b001;
     end
  else if(fpga_pgm_cnt == 2'd2)
     begin
     pgm_led  <= 3'b011;
     fpga_pgm <= 3'b010;
     end
  else                                         
     begin
     pgm_led  <= pgm_led;                      
     fpga_pgm <= fpga_pgm;           
     end
end

endmodule
//------------------------------------------------------------------------------
// end of file
//------------------------------------------------------------------------------