// =============================================================================
// File Name      : tsd_read.v
// Module         : tsd_read
// Function       : tsd_read 
// Type           : RTL
// -----------------------------------------------------------------------------
// Update History :
// -----------------------------------------------------------------------------
// =============================================================================
// End Revision
// =============================================================================

// synthesis translate_off
`timescale 1ps / 1ps
// synthesis translate_on

module tsd_read #
(
    parameter    OVERTEMP1_ASSERT_THRESHOLD   = 8'h3C,
    parameter    OVERTEMP1_DEASSERT_THRESHOLD = 8'h37,
    parameter    OVERTEMP2_ASSERT_THRESHOLD   = 8'h64,
    parameter    OVERTEMP2_DEASSERT_THRESHOLD = 8'h5F
)(
    input               sys_clk,             //50MHz
    input               sys_rst_n,
     
    input               i2c_sda_in,
	output              i2c_sda_out,
    output              i2c_scl,
    output  reg         over_tempn1,
    output  reg         over_tempn2,
    input       [ 2:0]  channel_no,
    output  reg [31:0]  channel_data  
);

parameter     SLAVE_ADDR  = 7'h55;      
//pullup      (i2c_sda);
//pullup      (i2c_scl);
reg           cmd_en;
reg           r_cmd_en;
reg  [ 2:0]   r_cmd_cnt;
reg  [ 7:0]   r_initial;
wire          read_valid;
reg  [15:0]   reg_addr;
wire [31:0]   read_data;
reg  [31:0]   channel0;
reg  [31:0]   channel1;
reg  [31:0]   channel2;
reg  [31:0]   channel3;
reg  [31:0]   channel4;
reg  [31:0]   channel5;
reg  [31:0]   channel6;
reg  [31:0]   channel7;
reg  [15:0]   delay_cnt;
reg  [ 3:0]   overtemp1_cnt;
reg  [ 3:0]   overtemp2_cnt;
reg  [ 3:0]   undertemp1_cnt;
reg  [ 3:0]   undertemp2_cnt;

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_initial    <=  8'b0;
    end else begin
        r_initial    <=  {r_initial[6:0], 1'b1};
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_cmd_cnt    <=  3'b0;
    end else if (read_valid == 1'b1) begin
        r_cmd_cnt    <=  r_cmd_cnt + 1'b1;  
    end
end   

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        cmd_en    <=  1'b0;
    end else if (r_initial[6] && (!r_initial[7])) begin
        cmd_en    <=  1'b1;
    end else if (delay_cnt == 16'hFFFF) begin
        cmd_en    <=  1'b1;
    end else begin
        cmd_en    <=  1'b0;
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        delay_cnt    <=  16'b0;
    end else if (read_valid) begin
        delay_cnt    <=  16'h1;
    end else if (delay_cnt != 16'b0) begin
        delay_cnt    <=  delay_cnt + 1'b1;
    end
end  

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        reg_addr    <=  16'b0;
    end else if (cmd_en) begin
        case(r_cmd_cnt)
            3'd0 : begin reg_addr <= 16'h0000; end
            3'd1 : begin reg_addr <= 16'h0010; end
            3'd2 : begin reg_addr <= 16'h0020; end
            3'd3 : begin reg_addr <= 16'h0030; end
            3'd4 : begin reg_addr <= 16'h0040; end 
            3'd5 : begin reg_addr <= 16'h0050; end 
            3'd6 : begin reg_addr <= 16'h0060; end 
            3'd7 : begin reg_addr <= 16'h0070; end       
        endcase 
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_cmd_en    <=  1'b0;
    end else begin
        r_cmd_en    <=  cmd_en;
    end
end    

i2c_read i2c_read_i
(
    .sys_clk    (sys_clk   ),             //50MHz
    .sys_rst_n  (sys_rst_n ), 
    .i2c_sda_in (i2c_sda_in),
	.i2c_sda_out(i2c_sda_out),
    .i2c_scl    (i2c_scl   ),    
    .reg_addr   (reg_addr  ),
    .cmd_en     (r_cmd_en  ),
    .read_data  (read_data ),
    .read_valid (read_valid)        
);

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        channel0    <=  32'b0;
        channel1    <=  32'b0;
        channel2    <=  32'b0;
        channel3    <=  32'b0;
        channel4    <=  32'b0;
        channel5    <=  32'b0;
        channel6    <=  32'b0;
        channel7    <=  32'b0;    
    end else if (read_valid) begin
        case(r_cmd_cnt)
            3'd0 : begin channel0 <= read_data; end
            3'd1 : begin channel1 <= read_data; end
            3'd2 : begin channel2 <= read_data; end
            3'd3 : begin channel3 <= read_data; end
            3'd4 : begin channel4 <= read_data; end 
            3'd5 : begin channel5 <= read_data; end 
            3'd6 : begin channel6 <= read_data; end 
            3'd7 : begin channel7 <= read_data; end      
        endcase 
    end
end   

always @(*) begin
    case(channel_no)
        3'd0 : begin channel_data <= channel0; end
        3'd1 : begin channel_data <= channel1; end
        3'd2 : begin channel_data <= channel2; end
        3'd3 : begin channel_data <= channel3; end
        3'd4 : begin channel_data <= channel4; end 
        3'd5 : begin channel_data <= channel5; end
        3'd6 : begin channel_data <= channel6; end 
        3'd7 : begin channel_data <= channel7; end    
    endcase 
end 

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        overtemp1_cnt   <=  4'b0;
        overtemp2_cnt   <=  4'b0;   
        undertemp1_cnt  <=  4'b0;
        undertemp2_cnt  <=  4'b0;		
    end else if (read_valid && (r_cmd_cnt == 3'd7) ) begin
        if (( over_tempn1 == 1'b1 ) && (read_data[15:8] >= OVERTEMP1_ASSERT_THRESHOLD) && (read_data[31] == 1'b0)) begin
            overtemp1_cnt <= overtemp1_cnt + 1'b1;     
        end else begin
            overtemp1_cnt <=  4'b0;
        end
		
        if (( over_tempn1 == 1'b0 ) && (read_data[15:8] < OVERTEMP1_DEASSERT_THRESHOLD) && (read_data[31] == 1'b0)) begin
            undertemp1_cnt <= undertemp1_cnt + 1'b1;     
        end else begin
            undertemp1_cnt <=  4'b0;
        end
		
        if (( over_tempn2 == 1'b1 ) && (read_data[15:8] >= OVERTEMP2_ASSERT_THRESHOLD) && (read_data[31] == 1'b0)) begin
            overtemp2_cnt <= overtemp2_cnt + 1'b1;     
        end else begin
            overtemp2_cnt <=  4'b0;
        end
		
        if (( over_tempn2 == 1'b0 ) && (read_data[15:8] < OVERTEMP2_DEASSERT_THRESHOLD) && (read_data[31] == 1'b0)) begin
            undertemp2_cnt <= undertemp2_cnt + 1'b1;     
        end else begin
            undertemp2_cnt <=  4'b0;
        end
    end
end 
  
always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        over_tempn1  <=  1'b1;
        over_tempn2  <=  1'b1;   
    end else if (read_valid && (r_cmd_cnt == 3'd7) ) begin
        if (undertemp1_cnt == 4'b1111) begin
            over_tempn1 <= 1'b1;
        end else if (overtemp1_cnt == 4'b1111) begin
            over_tempn1 <= 1'b0;
        end
    
        if (undertemp2_cnt == 4'b1111) begin
            over_tempn2 <= 1'b1;
        end else if (overtemp2_cnt == 4'b1111) begin
            over_tempn2 <= 1'b0;
        end
    end
end
  
endmodule              