// synthesis translate_off
`timescale 1ps / 1ps
// synthesis translate_on

module config_ltm2987
(
    input         sys_clk,             //50MHz
    input         sys_rst_n,
     
    output        i2c_sda,
    output        i2c_scl,
    output        set_done    
);      

reg         write_en;
reg [ 1:0]  r_cmd_cnt;
reg [ 7:0]  r_initial = 8'b0;
wire        write_ready;
reg [ 7:0]  reg_addr;
reg [ 7:0]  reg_data;
reg         r_set_done;
reg         r_write_en;
  
always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_initial   <=  8'b0;
    end else begin
        r_initial   <=  {r_initial[6:0], 1'b1};
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_cmd_cnt   <=  2'b0;
    end else if (write_ready == 1'b1) begin
        r_cmd_cnt   <=  r_cmd_cnt + 1'b1;
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_set_done  <=  1'b0;
    end else if ((r_cmd_cnt == 2'd3) && write_ready)  begin
        r_set_done  <=  1'b1;
    end
end    

assign set_done = r_set_done;    

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        write_en    <=  1'b0;
    end else if (r_initial[6] && (!r_initial[7])) begin
        write_en    <=  1'b1;
    end else if (write_ready && (r_cmd_cnt != 2'd3)) begin
        write_en    <=  1'b1;
    end else begin
        write_en    <=  1'b0;
    end
end  

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_write_en  <=  1'b0;
    end else begin
        r_write_en  <=  write_en;
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        reg_addr    <=  8'b0;
        reg_data    <=  8'b0;
    end else if (r_initial[6] && (!r_initial[7])) begin
        reg_addr    <=  8'h00;
        reg_data    <=  8'hFF;                                         //SET PAGE to 0xFF                                              
    end else if (write_ready) begin
        case(r_cmd_cnt)
            2'd0  : begin reg_addr <= 8'h01; reg_data <= 8'h40; end    //SEQUENCE OFF 2987
            2'd1  : begin reg_addr <= 8'h00; reg_data <= 8'hFF; end    //SET PAGE to 0xFF
            2'd2  : begin reg_addr <= 8'h01; reg_data <= 8'h40; end    //SEQUENCE OFF 2987
        endcase 
    end
end    

i2c_write #
(
   .SLAVE_ADDR (7'h5C      )
) i2c_write_i
(
   .sys_clk    (sys_clk    ),             //50MHz
   .sys_rst_n  (sys_rst_n  ), 
   .i2c_sda    (i2c_sda    ),
   .i2c_scl    (i2c_scl    ),    
   .reg_addr   (reg_addr   ),
   .reg_data   (reg_data   ),
   .write_en   (r_write_en ),
   .write_ready(write_ready)        
);
       
endmodule              