// synthesis translate_off
`timescale 1ps / 1ps
// synthesis translate_on

module i2c_write #
(
    parameter    SLAVE_ADDR = 7'h4C
)
(
    input         sys_clk,             //50MHz
    input         sys_rst_n,
     
    output        i2c_sda,
    output        i2c_scl,
    
    input  [ 7:0] reg_addr,
    input  [ 7:0] reg_data,
    input         write_en,
    output        write_ready        
);

//parameter      SLAVE_ADDR  = 7'h4C;      
  
parameter      IDLE        = 3'b000,
               I2C_START   = 3'b001,
               I2C_DATA    = 3'b010,
               I2C_END     = 3'b100;

reg  [ 2:0]  next_gen_state;
reg  [ 2:0]  gen_state;

reg  [ 8:0]  r_divide_clk;
reg          r1_divide_clk;
wire         slow_clk;
reg  [ 1:0]  r_cnt;
reg  [ 4:0]  r_data_cnt;
reg          r_sda;
reg          r_scl;
reg  [ 7:0]  r_reg_data;
reg  [ 7:0]  r_reg_addr;
reg          r_write_ready;
reg  [ 7:0]  r_ready_cnt;

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_reg_data     <=  8'b0;
        r_reg_addr     <=  8'b0;
    end else if (write_en == 1'b1) begin
        r_reg_data     <=  reg_data;
        r_reg_addr     <=  reg_addr;
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_divide_clk   <=  9'b0;
        r1_divide_clk  <=  1'b0;
    end else begin
        r_divide_clk   <= r_divide_clk + 1'b1;
        r1_divide_clk  <= r_divide_clk[8];
    end
end

assign slow_clk = r1_divide_clk^r_divide_clk[8];     

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_data_cnt     <=  5'b0;
    end else if (gen_state == IDLE) begin
        r_data_cnt     <=  5'b0;
    end else if (slow_clk &&(r_cnt == 3'b11) && (gen_state == I2C_DATA)) begin
        r_data_cnt     <= r_data_cnt + 1'b1;
    end
end    

always @( * ) begin
    next_gen_state = gen_state;
    case (gen_state)
        IDLE : begin
            if (write_en)
                next_gen_state = I2C_START;
        end
        I2C_START: begin
            if (slow_clk && (r_cnt == 3'b11)) begin
                next_gen_state = I2C_DATA;
            end   
        end  
        I2C_DATA:begin
            if (r_data_cnt == 5'd27) begin          
                next_gen_state = I2C_END;
            end 
        end
        I2C_END:begin
            if (slow_clk && (r_cnt == 3'b11)) begin          
                next_gen_state = IDLE;
            end    
        end      
        default : begin
            next_gen_state = IDLE;
        end
    endcase
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (~sys_rst_n) begin
        gen_state <= IDLE;
    end else begin
        gen_state <= next_gen_state;
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_cnt     <=  2'b0;
    end else if (gen_state == IDLE) begin
        r_cnt     <=  2'b0;      
    end else if (slow_clk) begin
        r_cnt     <= r_cnt + 1'b1;
    end
end 

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_scl       <=  1'b1;      
        r_sda       <=  1'b1;
    end else if (gen_state == I2C_START) begin
        if (slow_clk && (r_cnt == 3'b0)) begin
            r_sda   <= 1'b0;  
        end else if(slow_clk && (r_cnt == 3'b10))begin
            r_scl   <= 1'b0;
        end               
    end else if (gen_state == I2C_DATA) begin
        if (slow_clk && (r_cnt == 3'b0)) begin
            case(r_data_cnt)
                5'd0 : r_sda <= SLAVE_ADDR[6];
                5'd1 : r_sda <= SLAVE_ADDR[5];
                5'd2 : r_sda <= SLAVE_ADDR[4];
                5'd3 : r_sda <= SLAVE_ADDR[3];
                5'd4 : r_sda <= SLAVE_ADDR[2];
                5'd5 : r_sda <= SLAVE_ADDR[1];
                5'd6 : r_sda <= SLAVE_ADDR[0];
                5'd7 : r_sda <= 1'b0;
                5'd8 : r_sda <= 1'b1;
                5'd9 : r_sda <= r_reg_addr[7];
                5'd10: r_sda <= r_reg_addr[6];
                5'd11: r_sda <= r_reg_addr[5];
                5'd12: r_sda <= r_reg_addr[4];
                5'd13: r_sda <= r_reg_addr[3];
                5'd14: r_sda <= r_reg_addr[2];
                5'd15: r_sda <= r_reg_addr[1];
                5'd16: r_sda <= r_reg_addr[0];
                5'd17: r_sda <= 1'b1;  
                5'd18: r_sda <= r_reg_data[7];
                5'd19: r_sda <= r_reg_data[6];
                5'd20: r_sda <= r_reg_data[5];
                5'd21: r_sda <= r_reg_data[4];
                5'd22: r_sda <= r_reg_data[3];
                5'd23: r_sda <= r_reg_data[2];
                5'd24: r_sda <= r_reg_data[1];
                5'd25: r_sda <= r_reg_data[0];
                5'd26: r_sda <= 1'b1;            
                default : r_sda <= 1'b1;        
            endcase        
        end
        if (slow_clk && (r_cnt == 3'b01)) begin
            r_scl   <= 1'b1;
        end else if (slow_clk && (r_cnt == 3'b11)) begin
            r_scl   <= 1'b0;
        end        
    end else if (gen_state == I2C_END) begin
        if (slow_clk && (r_cnt == 3'b01)) begin
            r_scl   <= 1'b1;  
        end else if(slow_clk && (r_cnt == 3'b11))begin
            r_sda   <= 1'b1;
        end else if(slow_clk && (r_cnt == 3'b0))begin
            r_sda   <= 1'b0;          
        end    
    end
end

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_write_ready     <=  1'b0;    
    end else if ((gen_state == I2C_END) && slow_clk && (r_cnt == 3'b11)) begin
        r_write_ready     <=  1'b1;
    end else if ( slow_clk && (r_ready_cnt == 8'd32)) begin
        r_write_ready     <=  1'b0;        
    end
end   

always @(posedge sys_clk or negedge sys_rst_n) begin
    if (sys_rst_n == 1'b0) begin
        r_ready_cnt     <=  8'b0;    
    end else if (r_write_ready) begin
        if (slow_clk) begin
            r_ready_cnt <=  r_ready_cnt + 1'b1; 
        end
    end else begin
        r_ready_cnt     <=  8'b0;
    end
end    

assign write_ready = slow_clk & (r_ready_cnt == 8'd32);

assign i2c_sda = r_sda;
assign i2c_scl = r_scl;
        
endmodule              