//
// fixed for 9.1 jan 21 2010 cruben
//
`include "timescale.v"
`include "i2c_master_defines.v"

module opencores_i2c
(
  wb_clk_i, wb_rst_i, wb_adr_i, wb_dat_i, wb_dat_o,
  wb_we_i, wb_stb_i, /*wb_cyc_i,*/ wb_ack_o, wb_inta_o,
  scl_pad_io, sda_pad_io, slow_clock, overtempn
);


// Common bus signals
input        wb_clk_i;    // WISHBONE clock
input        wb_rst_i;    // WISHBONE reset

// Slave signals
input  [2:0] wb_adr_i;    // WISHBONE address input
input  [7:0] wb_dat_i;    // WISHBONE data input
output [7:0] wb_dat_o;    // WISHBONE data output
input        wb_we_i;    // WISHBONE write enable input
input        wb_stb_i;    // WISHBONE strobe input
//input        wb_cyc_i;    // WISHBONE cycle input
output       wb_ack_o;    // WISHBONE acknowledge output
output       wb_inta_o;   // WISHBONE interrupt output

// I2C signals
inout        scl_pad_io;  // I2C clock io
inout        sda_pad_io;  // I2C data io
input        slow_clock;
input        overtempn;

wire         wb_cyc_i;    // WISHBONE cycle input

wire scl_1619;
wire sda_1619;
wire scl_2987;
wire sda_2987;
wire set_done;

wire scl_padoen;
wire sda_padoen;
reg [14:0] temp_mon_cnt = 15'b0;
reg  ltm2987_access_en = 1'b0;

config_max1619 config_max1619_i
(
    .sys_clk  (wb_clk_i),             //50MHz
    .slow_clk (slow_clock),
    .sys_rst_n(!wb_rst_i),  
    .i2c_sda  (sda_1619),
    .i2c_scl  (scl_1619),
    .set_done (set_done)   
);
   
config_ltm2987 config_ltm2987_i
(
    .sys_clk  (wb_clk_i),             //50MHz
    .sys_rst_n(ltm2987_access_en),  
    .i2c_sda  (sda_2987),
    .i2c_scl  (scl_2987),
    .set_done ()   
);  

always @(posedge slow_clock or posedge wb_rst_i) begin
    if (wb_rst_i) begin
        temp_mon_cnt   <=  15'b0;
    end else if (overtempn == 1'b0) begin
        temp_mon_cnt   <=  temp_mon_cnt + 1'b1;
    end else begin
        temp_mon_cnt   <=  15'b0;
    end
end   

always @(posedge slow_clock) begin
    if (&temp_mon_cnt == 1'b1) begin
        ltm2987_access_en   <=  1'b1;
    end
end

// Wire tri-state scl/sda
wire scl_pad_i;
wire scl_pad_o;
wire scl_pad_io;
wire scl_padoen_o;

assign wb_cyc_i = wb_stb_i;
assign scl_pad_i = scl_pad_io;
assign scl_pad_io = scl_padoen ? 1'bZ : 1'b0;
assign scl_padoen = ltm2987_access_en ? scl_2987 : (set_done ? scl_padoen_o : scl_1619);

wire sda_pad_i;
wire sda_pad_o;
wire sda_pad_io;
wire sda_padoen_o;

assign sda_pad_i = sda_pad_io;
assign sda_pad_io = sda_padoen ? 1'bZ : 1'b0;
assign sda_padoen = ltm2987_access_en ? sda_2987 : (set_done ? sda_padoen_o : sda_1619);

// Avalon doesn't have an asynchronous reset
//  set it to be inactive and just use synchronous reset
//  reset level is a parameter, 0 is the default (active-low reset)
wire arst_i;

assign arst_i = 1'b1;

// Connect the top level I2C core
i2c_master_top i2c_master_top_inst
(
  .wb_clk_i(wb_clk_i), .wb_rst_i(wb_rst_i), .arst_i(arst_i),
  
  .wb_adr_i(wb_adr_i), .wb_dat_i(wb_dat_i), .wb_dat_o(wb_dat_o),
  .wb_we_i(wb_we_i), .wb_stb_i(wb_stb_i), .wb_cyc_i(wb_cyc_i),
  .wb_ack_o(wb_ack_o), .wb_inta_o(wb_inta_o),
  
  .scl_pad_i(scl_pad_i), .scl_pad_o(scl_pad_o), .scl_padoen_o(scl_padoen_o),
  .sda_pad_i(sda_pad_i), .sda_pad_o(sda_pad_o), .sda_padoen_o(sda_padoen_o)
);

endmodule
