library ieee;
use ieee.std_logic_1164.all;

entity sys_max_id is
	port (
		reset_n: in std_logic;
		clk : in std_logic;
		
		version : in std_logic_vector(15 downto 0);
		project_id : in std_logic_vector(15 downto 0);
		diff_raw : in std_logic_vector(23 downto 0);
		single_raw : in std_logic_vector(23 downto 0);
		fpga_pgm : in std_logic_vector(2 downto 0);
		page_sel_pb : in std_logic;
		cpu_resetn : out std_logic; 
		fpga_reconfign : out std_logic;
		page_sel : out std_logic_vector(3 downto 0);
		control_gui : out std_logic_vector(7 downto 0);
		temp_data : out std_logic_vector(7 downto 0);
		i2c_inf_sw : out std_logic;
		
		-- interface for AvMM-slave
		slv_en_n : in std_logic;
		slv_address : in std_logic_vector(2 downto 0);
		slv_read_n : in std_logic;
		slv_write_n : in std_logic;
		slv_data_in : in  std_logic_vector(31 downto 0);
		slv_data_out : out std_logic_vector(31 downto 0);
		slv_waitrequest_n : out std_logic -- 0: under process   1:process done, same ad wait_n
	);
end sys_max_id;

architecture rtl of sys_max_id is
	signal trigger, trigger_d1, trigger_d2, trigger_pls : std_logic;
	signal cpu_resetn_int : std_logic;
	signal fpga_reconfign_int : std_logic;
	signal page_sel_int : std_logic_vector (3 downto 0);
	signal control_gui_int : std_logic_vector (7 downto 0);
	signal temp_data_int : std_logic_vector (7 downto 0);
	signal i2c_inf_sw_int : std_logic;

begin

    cpu_resetn     <= cpu_resetn_int; 
    fpga_reconfign <= fpga_reconfign_int;
    page_sel       <= page_sel_int;
    control_gui    <= control_gui_int;
    temp_data      <= temp_data_int;
    i2c_inf_sw     <= i2c_inf_sw_int;

	process(clk, reset_n)begin
		if(reset_n = '0')then
			slv_data_out <= x"00000000";
		elsif(clk'event and clk = '1')then
			if(slv_en_n = '0' and slv_read_n = '0')then
				if(slv_address = "000")then
					slv_data_out <= x"0000" & version;
				elsif(slv_address = "001")then
					slv_data_out <= x"0000" & project_id;
				elsif(slv_address = "010")then
					slv_data_out <= x"00" & diff_raw;
				elsif(slv_address = "011")then
					slv_data_out <= x"00" & single_raw;
				elsif(slv_address = "100")then
					slv_data_out <= x"0000000" & page_sel_int(3) & fpga_pgm;
				elsif(slv_address = "101")then
					slv_data_out <= x"000000" & control_gui_int;
				elsif(slv_address = "110")then
					slv_data_out <= x"000000" & temp_data_int;
				elsif(slv_address = "111")then
					slv_data_out <= x"0000000" & "000" & i2c_inf_sw_int;
				else
					slv_data_out <= x"00000000";
				end if;
			end if;
		end if;
	end process;
	
	process(clk, reset_n)begin
		if(reset_n = '0')then
            cpu_resetn_int <= '1';
            fpga_reconfign_int <= '1';
            page_sel_int <= "0000";
			control_gui_int <= x"00";
            temp_data_int <= x"30";
            i2c_inf_sw_int <= '0';
		elsif(clk'event and clk = '1')then
			if(slv_en_n = '0' and slv_write_n = '0')then
				if(slv_address = "010")then
					cpu_resetn_int <= slv_data_in(0);
				elsif(slv_address = "011")then
					fpga_reconfign_int <= slv_data_in(0);
				elsif(slv_address = "100")then
					page_sel_int <= slv_data_in(3 downto 0);
				elsif(slv_address = "101")then
					control_gui_int <= slv_data_in(7 downto 0);
				elsif(slv_address = "110")then
					temp_data_int <= slv_data_in(7 downto 0);
				elsif(slv_address = "111")then
					i2c_inf_sw_int <= slv_data_in(0);
				end if;
		    elsif (page_sel_pb = '1') then
				page_sel_int(3) <= '0';		
			end if;
		end if;
	end process;
	
	process(clk, reset_n)begin
		if(reset_n = '0')then
			trigger <= '0';
		elsif(clk'event and clk = '1')then
			if(slv_en_n = '0' and (slv_read_n = '0' or slv_write_n = '0'))then
				trigger <= '1';
			else
				trigger <= '0';
			end if;
		end if;
	end process;
	
	process(clk, reset_n)begin
		if(reset_n = '0')then
			trigger_d1 <= '0';
			trigger_d2 <= '1';
		elsif(clk'event and clk = '1')then
			trigger_d2 <= trigger_d1; -- trigger_d1 <= trigger_d2;
			trigger_d1 <= not trigger;
		end if;
	end process;
	
	trigger_pls <= trigger and trigger_d2;
	
	
	process(clk, reset_n)begin
		if(reset_n = '0')then
			slv_waitrequest_n <= '1';
		elsif(clk'event and clk = '1')then
			if(trigger_pls = '1')then
				slv_waitrequest_n <= '1';
			elsif(slv_en_n = '1' or slv_read_n = '1')then
				slv_waitrequest_n <= '0';
			end if;
		end if;
	end process;
	
	
end rtl;
